<?xml version="1.0" encoding="UTF-8"?>
<!-- 
  Copyright 2013 The Language Archive, Max Planck Institute for Psycholinguistics (MPIP)

  This code is part of ISOcat (www.isocat.org). 
  
  ISOcat is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 3 as
  published by the Free Software Foundation.

  ISOcat is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with ISOcat. If not, see <http://www.gnu.org/licenses/>.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0" xmlns:dcif="http://www.isocat.org/ns/dcif" xmlns:dcr="http://www.isocat.org/ns/dcr.rdf#" xmlns:isocat="http://www.isocat.org/" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#" xmlns:skos="http://www.w3.org/2004/02/skos/core#" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:dcterms="http://purl.org/dc/terms/" xmlns:openskos="http://openskos.org/xmlns#" exclude-result-prefixes="dcif isocat">
    <xsl:output method="xml" encoding="utf-8"/>
    <xsl:param name="base"/>
    <xsl:param name="props"/>
	<xsl:param name="approve" select="('CLARIN-NL/VL','Athens Core')"/>
	<xsl:variable name="doc" select="/"/>
    <xsl:template match="text()"/>
    <xsl:template match="dcif:dataCategorySelection">
        <rdf:RDF>
            <xsl:if test="normalize-space($base)!=''">
                <xsl:attribute name="xml:base" select="concat($base,'#')"/>
            </xsl:if>
        	<openskos:skosCollection rdf:about="tmp:S-{replace(@owner,'\s','')}-{replace(@name,'\s','')}">
        		<dc:title>
        			<xsl:value-of select="@name"/>
        		</dc:title>
        		<xsl:for-each select="dcif:dataCategory">
        			<skos:member rdf:resource="tmp:C-{substring-after(@pid,'DC-')}"/>
        		</xsl:for-each>
        	</openskos:skosCollection>
            <xsl:for-each select="distinct-values(//dcif:profile)">
            	<xsl:variable name="prof" select="."/>
                <skos:ConceptScheme rdf:about="tmp:P-{replace(.,'\s','')}">
                    <dc:title>
                        <xsl:value-of select="."/>
                    </dc:title>
                	<xsl:for-each select="$doc//dcif:dataCategory[dcif:descriptionSection/dcif:profile=$prof]">
                		<skos:hasTopConcept rdf:resource="tmp:C-{substring-after(@pid,'DC-')}"/>
                	</xsl:for-each>
                </skos:ConceptScheme>
            </xsl:for-each>
            <xsl:apply-templates/>
        </rdf:RDF>
    </xsl:template>
    <xsl:template match="dcif:dataCategory">
        <xsl:variable name="name" select="@isocat:name"/>
        <xsl:variable name="concept">
        	<skos:Concept rdf:about="tmp:C-{substring-after(@pid,'DC-')}">
        		<xsl:choose>
        			<xsl:when test="dcif:registrationStatus=('deprecated','superseded')">
        				<openskos:status>expired</openskos:status>
        			</xsl:when>
        			<xsl:when test="isocat:recommendedBy=$approve">
        				<openskos:status>approved</openskos:status>
        			</xsl:when>
        			<xsl:otherwise>
        				<openskos:status>candidate</openskos:status>
        			</xsl:otherwise>
        		</xsl:choose>
                <!-- changeNote to satisfy the CC-BY license -->
                <skos:changeNote>
                    <xsl:text>This concept is based on the ISOcat data category: </xsl:text>
                    <xsl:value-of select="@pid"/>
                </skos:changeNote>

                <!-- administration section:
                dcif:identifier -> skos:notation
                dcif:version
                dcif:registrationStatus
                dcif:origin -> skos:note (DELETE?)
                dcif:justification -> skos:note (DELETE?)
                dcif:explanatoryComment -> skos:editorialNote (DELETE?)
                dcif:unresolvedIssue 
                dcif:effectiveDate
                dcif:untilDate
                dcif:creation -> skos:changeNote (DELETE)
                dcif:lastChange -> skos:changeNote (DELETE)
                -->

                <!-- xsl:for-each to change context, but there will be only one Administration Record -->
                <xsl:for-each select="dcif:administrationInformationSection/dcif:administrationRecord">
                    <skos:notation>
                        <xsl:value-of select="dcif:identifier"/>
                    </skos:notation>
                </xsl:for-each>

                <!-- xsl:for-each to change context, but there will be only one Description Section -->
                <xsl:for-each select="dcif:descriptionSection">

                    <!-- description section:
                    dcif:profile -> skos:inScheme
                    dcif:dataElementName -> skos:notation
                    -->
                    <xsl:for-each select="dcif:profile">
                        <skos:inScheme rdf:resource="tmp:P-{replace(.,'\s','')}"/>
                    </xsl:for-each>
                    
                    <xsl:for-each-group select="dcif:dataElementNameSection" group-by="dcif:dataElementName">
                        <xsl:if test="normalize-space(current-grouping-key())!=''">
                            <skos:notation>
                                <xsl:value-of select="normalize-space(current-grouping-key())"/>
                            </skos:notation>
                        </xsl:if>
                    </xsl:for-each-group>

                    <!-- language sections:
                    dcif:name (preferred) -> skos:prefLabel
                    dcif:name (standardized) -> skos:prefLabel
                    dcif:name (admitted) -> skos:altLabel
                    dcif:name (deprecated) -> skos:hiddenLabel
                    dcif:name (superseded) -> skos:hiddenLabel
                    dcif:definition -> skos:definition
                    dcif:example -> skos:example
                    dcif:explanation -> skos:scopeNote
                    dcif:note -> skos:note
                    -->

                    <xsl:for-each select="dcif:languageSection[dcif:language='en']">
                        <xsl:variable name="lang" select="dcif:language"/>

                        <xsl:if test="$lang='en'">
                            <skos:prefLabel xml:lang="{$lang}">
                                <xsl:value-of select="normalize-space($name)"/>
                            </skos:prefLabel>
                        </xsl:if>

                        <xsl:for-each-group select="dcif:nameSection[not($lang='en' and dcif:name=$name)]" group-by="dcif:nameStatus">
                            <xsl:choose>
                                <xsl:when test="current-grouping-key()='preferred name'">
                                    <xsl:variable name="pref" select="if ($lang='en') then () else (current-group()[1])"/>
                                    <xsl:if test="exists($pref)">
                                        <skos:prefLabel xml:lang="{$lang}">
                                            <xsl:value-of select="normalize-space(dcif:name)"/>
                                        </skos:prefLabel>
                                    </xsl:if>
                                    <xsl:for-each select="current-group()[dcif:name ne $pref/dcif:name]">
                                        <skos:altLabel xml:lang="{$lang}">
                                            <xsl:value-of select="normalize-space(dcif:name)"/>
                                        </skos:altLabel>
                                    </xsl:for-each>
                                </xsl:when>
                                <xsl:when test="current-grouping-key()='standardized name'">
                                    <xsl:for-each select="current-group()">
                                        <skos:altLabel xml:lang="{$lang}">
                                            <xsl:value-of select="normalize-space(dcif:name)"/>
                                        </skos:altLabel>
                                    </xsl:for-each>
                                </xsl:when>
                                <xsl:when test="current-grouping-key()='admitted name'">
                                    <xsl:for-each select="current-group()">
                                        <skos:altLabel xml:lang="{$lang}">
                                            <xsl:value-of select="normalize-space(dcif:name)"/>
                                        </skos:altLabel>
                                    </xsl:for-each>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:for-each select="current-group()">
                                        <skos:hiddenLabel xml:lang="{$lang}">
                                            <xsl:value-of select="normalize-space(dcif:name)"/>
                                        </skos:hiddenLabel>
                                    </xsl:for-each>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:for-each-group>

                        <xsl:for-each select="dcif:definitionSection">
                            <skos:definition xml:lang="{$lang}">
                                <xsl:value-of select="normalize-space(dcif:definition)"/>
                                <xsl:if test="normalize-space(dcif:source)!=''">
                                    <xsl:text> (source: </xsl:text>
                                    <xsl:value-of select="normalize-space(dcif:source)"/>
                                    <xsl:text>)</xsl:text>
                                </xsl:if>
                            </skos:definition>
                        </xsl:for-each>

                        <xsl:for-each select="dcif:exampleSection">
                            <skos:example xml:lang="{$lang}">
                                <xsl:value-of select="normalize-space(dcif:example)"/>
                                <xsl:if test="normalize-space(dcif:source)!=''">
                                    <xsl:text> (source: </xsl:text>
                                    <xsl:value-of select="normalize-space(dcif:source)"/>
                                    <xsl:text>)</xsl:text>
                                </xsl:if>
                            </skos:example>
                        </xsl:for-each>

                        <xsl:for-each select="dcif:explanationSection">
                            <skos:scopeNote xml:lang="{$lang}">
                                <xsl:value-of select="normalize-space(dcif:explanation)"/>
                                <xsl:if test="normalize-space(dcif:source)!=''">
                                    <xsl:text> (source: </xsl:text>
                                    <xsl:value-of select="normalize-space(dcif:source)"/>
                                    <xsl:text>)</xsl:text>
                                </xsl:if>
                            </skos:scopeNote>
                        </xsl:for-each>

                        <xsl:for-each select="dcif:note">
                            <skos:note xml:lang="{$lang}">
                                <xsl:value-of select="normalize-space(.)"/>
                            </skos:note>
                        </xsl:for-each>

                    </xsl:for-each>

                </xsl:for-each>

                <!-- conceptual domains:            
                isA -> skos:broader (DELETE)
                -->

                <!-- memberships:
                is in a closed value domain -> skos:inScheme (DELETE)
                -->

            </skos:Concept>
        </xsl:variable>
        <xsl:apply-templates select="$concept" mode="cleanup"/>
    </xsl:template>
    <xsl:template match="@*|node()" mode="cleanup">
        <xsl:copy>
            <xsl:apply-templates select="@*|node()" mode="#current"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="skos:notation" mode="cleanup">
        <xsl:if test="not(preceding-sibling::skos:notation=current())">
            <xsl:copy>
                <xsl:apply-templates select="@*|node()" mode="#current"/>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    <xsl:template match="skos:prefLabel" mode="cleanup">
        <xsl:if test="not(preceding-sibling::skos:prefLabel=current())">
            <xsl:copy>
                <xsl:apply-templates select="@*|node()" mode="#current"/>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    <xsl:template match="skos:altLabel" mode="cleanup">
        <xsl:if test="not(preceding-sibling::skos:altLabel=current())">
            <xsl:copy>
                <xsl:apply-templates select="@*|node()" mode="#current"/>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    <xsl:template match="skos:hiddenLabel" mode="cleanup">
        <xsl:if test="not(preceding-sibling::skos:hiddenLabel=current())">
            <xsl:copy>
                <xsl:apply-templates select="@*|node()" mode="#current"/>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
</xsl:stylesheet>
